<?php

namespace App\Nova\Actions;

use App\Models\User;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Support\Collection;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\Validator;
use Laravel\Nova\Actions\Action;
use Laravel\Nova\Actions\ActionResponse;
use Laravel\Nova\Fields\ActionFields;
use Laravel\Nova\Fields\File;
use Laravel\Nova\Http\Requests\NovaRequest;
use Maatwebsite\Excel\Facades\Excel;

class ImportUsersAction extends Action implements ShouldQueue
{
    use InteractsWithQueue;
    use Queueable;

    /**
     * The path to the temporarily stored import file.
     * @var string|null
     */
    protected $filePath;

    /**
     * The displayable name of the action.
     * @var string
     */
    public $name = 'استيراد مستخدمين من CSV';

    /**
     * Perform the action on the given models.
     * @return mixed
     */
    public function handle(ActionFields $fields, Collection $models)
    {
        $file = $fields->file;

        if (!$file) {
            return Action::danger('الرجاء رفع ملف CSV.');
        }

        // 1. Store the file temporarily and get its path
        try {
            // Generate a unique name and store in 'temp_imports' directory within storage/app
            $this->filePath = $file->store('temp_imports');
        } catch (\Exception $e) {
            \Log::error('Failed to store temporary import file: ' . $e->getMessage());
            return Action::danger('فشل في تخزين الملف المؤقت.');
        }

        // Note: From this point on, this method might be executed by the queue worker.
        // It will have access to $this->filePath but not the original $file object.

        try {
            // 2. Read data from the stored file path
            $data = Excel::toArray(new \stdClass(), Storage::path($this->filePath)); // Use Storage::path()

            if (empty($data[0])) {
                 // Clean up before returning
                Storage::delete($this->filePath);
                return Action::danger('ملف CSV فارغ أو غير صالح.');
            }

            $importedCount = 0;
            $skippedCount = 0;
            $errors = [];
            $header = array_map('trim', array_map('strtolower', $data[0][0]));

            if (!in_array('name', $header) || !in_array('email', $header) || !in_array('password', $header)) {
                 // Clean up before returning
                 Storage::delete($this->filePath);
                 return Action::danger('ملف CSV يجب أن يحتوي على الأعمدة: name, email, password');
            }

            foreach (array_slice($data[0], 1) as $index => $row) {
                $rowData = array_combine($header, array_pad($row, count($header), null));
                $rowNumber = $index + 2;

                $validator = Validator::make($rowData, [
                    'name' => 'required|string|max:255',
                    'email' => 'required|email|max:254|unique:users,email',
                    'password' => 'required|string|min:8',
                    'is_active' => 'nullable|boolean',
                ]);

                if ($validator->fails()) {
                    $skippedCount++;
                    $errors[] = "السطر {$rowNumber}: " . implode(', ', $validator->errors()->all());
                    continue;
                }

                User::create([
                    'name' => $rowData['name'],
                    'email' => $rowData['email'],
                    'password' => Hash::make($rowData['password']),
                    'is_active' => filter_var($rowData['is_active'] ?? true, FILTER_VALIDATE_BOOLEAN),
                ]);
                $importedCount++;
            }

             $message = "تم استيراد {$importedCount} مستخدم بنجاح.";
            if ($skippedCount > 0) {
                $message .= " تم تخطي {$skippedCount} سجل بسبب أخطاء:";
                // Clean up before returning
                Storage::delete($this->filePath);
                return Action::danger($message . " " . implode("; ", array_slice($errors, 0, 3)));
            }

             // Clean up successful import
             Storage::delete($this->filePath);
            return Action::message($message);


        } catch (\Exception $e) {
            // Log the detailed error for debugging
             \Log::error('Error during CSV import processing: ' . $e->getMessage() . ' Stack: ' . $e->getTraceAsString());
            // Clean up failed import
             if ($this->filePath && Storage::exists($this->filePath)) {
                 Storage::delete($this->filePath);
             }
            return Action::danger('حدث خطأ غير متوقع أثناء معالجة الملف.'); // Keep message generic for user
        }
    }

    /**
     * Get the fields available on the action.
     * @return array<int, \Laravel\Nova\Fields\Field>
     */
    public function fields(NovaRequest $request): array
    {
        return [
            File::make('ملف CSV', 'file')
                ->rules('required', 'file', 'mimes:csv,txt')
                ->help('يجب أن يحتوي الملف على الأعمدة: name, email, password. يمكن إضافة عمود is_active (1 أو 0).'),
        ];
    }
}
