<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\File;
use Laravel\Nova\Fields\MorphTo;
use Laravel\Nova\Http\Requests\NovaRequest;

class Attachment extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\Attachment>
     */
    public static $model = \App\Models\Attachment::class;

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'original_name';

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'original_name',
    ];

    /**
     * Get the displayable label of the resource.
     *
     * @return string
     */
    public static function label(): string
    {
        return 'المرفقات';
    }

    /**
     * Get the displayable singular label of the resource.
     *
     * @return string
     */
    public static function singularLabel(): string
    {
        return 'مرفق';
    }

    /**
     * Get the fields displayed by the resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @return array
     */
    public function fields(NovaRequest $request)
    {
        return [
            ID::make()->sortable(),

            // Relationship to the parent model (e.g., Asset)
            MorphTo::make('Attachable')->types([
                Asset::class, // Add other models here if needed
            ])->searchable(),

            Text::make('اسم الملف الأصلي', 'original_name')
                ->exceptOnForms() // Display only, not editable
                ->sortable(),

            File::make('الملف', 'file_path')
                ->disk('public') // Use the public disk
                ->path('attachments') // Store files in storage/app/public/attachments
                ->storeOriginalName('original_name') // Store original filename in 'original_name' column
                ->store(function (Request $request, $model) {
                    // Set the disk attribute on the model
                    $model->disk = 'public'; 

                    // Define how to store the file and its name
                    $file = $request->file('file_path');
                    $hashName = sha1($file->getClientOriginalName() . time()) . '.' . $file->getClientOriginalExtension();
                    
                    // Store the file and return an array of the attributes to be filled
                    return [
                        'file_path' => $file->storeAs('attachments', $hashName, 'public'), // Store the file path
                        'original_name' => $file->getClientOriginalName(), // Store the original name
                    ];
                })
                ->prunable() // Delete file when resource is deleted
                ->rules('required', 'file', 'max:10240'), // Example validation: required, max 10MB
        ];
    }

    /**
     * Get the cards available for the resource.
     *
     * @return array<int, \Laravel\Nova\Card>
     */
    public function cards(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @return array<int, \Laravel\Nova\Filters\Filter>
     */
    public function filters(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @return array<int, \Laravel\Nova\Lenses\Lens>
     */
    public function lenses(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @return array<int, \Laravel\Nova\Actions\Action>
     */
    public function actions(NovaRequest $request): array
    {
        return [];
    }
}
