<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\BelongsToMany;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Select;
use Laravel\Nova\Fields\Slug;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Textarea;
use Laravel\Nova\Http\Requests\NovaRequest;

class Permission extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\Permission>
     */
    public static $model = \App\Models\Permission::class;

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'name';

    /**
     * ترتيب الموارد في القائمة الجانبية.
     *
     * @var int
     */
    public static $priority = 4;

    /**
     * تسمية المورد في القائمة الجانبية.
     *
     * @return string
     */
    public static function label(): string
    {
        return 'الصلاحيات';
    }

    /**
     * تسمية المورد في القائمة الجانبية (المفرد).
     *
     * @return string
     */
    public static function singularLabel(): string
    {
        return 'صلاحية';
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له برؤية مورد الصلاحيات.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public static function authorizedToViewAny(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('permissions.view');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له برؤية الصلاحية.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToView(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('permissions.view');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بإنشاء صلاحية.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public static function authorizedToCreate(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('permissions.create');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بتحديث الصلاحية.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToUpdate(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('permissions.edit');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بحذف الصلاحية.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToDelete(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('permissions.delete');
    }

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'name', 'slug', 'group',
    ];

    /**
     * Get the fields displayed by the resource.
     *
     * @return array<int, \Laravel\Nova\Fields\Field>
     */
    public function fields(NovaRequest $request): array
    {
        return [
            ID::make()->sortable(),

            Text::make('الاسم', 'name')
                ->sortable()
                ->rules('required', 'max:255'),

            Slug::make('الاسم المختصر', 'slug')
                ->from('name')
                ->rules('required', 'max:255')
                ->creationRules('unique:permissions,slug')
                ->updateRules('unique:permissions,slug,{{resourceId}}'),

            Textarea::make('الوصف', 'description')
                ->hideFromIndex(),

            Select::make('المجموعة', 'group')
                ->options([
                    'users' => 'المستخدمين',
                    'roles' => 'الأدوار',
                    'permissions' => 'الصلاحيات',
                    'system' => 'النظام',
                    'other' => 'أخرى'
                ])
                ->displayUsingLabels()
                ->sortable(),

            BelongsToMany::make('الأدوار', 'roles', Role::class),
        ];
    }

    /**
     * Get the cards available for the resource.
     *
     * @return array<int, \Laravel\Nova\Card>
     */
    public function cards(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @return array<int, \Laravel\Nova\Filters\Filter>
     */
    public function filters(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @return array<int, \Laravel\Nova\Lenses\Lens>
     */
    public function lenses(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @return array<int, \Laravel\Nova\Actions\Action>
     */
    public function actions(NovaRequest $request): array
    {
        return [];
    }
}
