<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\BelongsToMany;
use Laravel\Nova\Fields\Boolean;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Slug;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Textarea;
use Laravel\Nova\Http\Requests\NovaRequest;

class Role extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\Role>
     */
    public static $model = \App\Models\Role::class;

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'name';

    /**
     * ترتيب الموارد في القائمة الجانبية.
     *
     * @var int
     */
    public static $priority = 3;

    /**
     * تسمية المورد في القائمة الجانبية.
     *
     * @return string
     */
    public static function label(): string
    {
        return 'الأدوار';
    }

    /**
     * تسمية المورد في القائمة الجانبية (المفرد).
     *
     * @return string
     */
    public static function singularLabel(): string
    {
        return 'دور';
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له برؤية مورد الأدوار.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public static function authorizedToViewAny(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('roles.view');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له برؤية الدور.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToView(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('roles.view');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بإنشاء دور.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public static function authorizedToCreate(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('roles.create');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بتحديث الدور.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToUpdate(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('roles.edit');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بحذف الدور.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToDelete(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('roles.delete');
    }

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'name', 'description',
    ];

    /**
     * Get the fields displayed by the resource.
     *
     * @return array<int, \Laravel\Nova\Fields\Field>
     */
    public function fields(NovaRequest $request): array
    {
        return [
            ID::make()->sortable(),

            Text::make('الاسم', 'name')
                ->sortable()
                ->rules('required', 'max:255'),

            Slug::make('الاسم المختصر', 'slug')
                ->from('name')
                ->rules('required', 'max:255')
                ->creationRules('unique:roles,slug')
                ->updateRules('unique:roles,slug,{{resourceId}}'),

            Textarea::make('الوصف', 'description')
                ->hideFromIndex(),

            Boolean::make('نشط', 'is_active')
                ->default(true),

            BelongsToMany::make('الصلاحيات', 'permissions', Permission::class),

            BelongsToMany::make('المستخدمين', 'users', User::class),
        ];
    }

    /**
     * Get the cards available for the resource.
     *
     * @return array<int, \Laravel\Nova\Card>
     */
    public function cards(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @return array<int, \Laravel\Nova\Filters\Filter>
     */
    public function filters(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @return array<int, \Laravel\Nova\Lenses\Lens>
     */
    public function lenses(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @return array<int, \Laravel\Nova\Actions\Action>
     */
    public function actions(NovaRequest $request): array
    {
        return [];
    }
}
