<?php

namespace App\Nova;

use Illuminate\Http\Request;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\HasMany;
use Laravel\Nova\Fields\BelongsTo;
use Laravel\Nova\Http\Requests\NovaRequest;
use Illuminate\Database\Eloquent\Builder;

class School extends Resource
{
    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\School>
     */
    public static $model = \App\Models\School::class;

    /**
     * The logical group associated with the resource.
     *
     * @var string
     */
    public static $group = 'إدارة المدارس';

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'name';

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'name',
    ];

    public static function label(): string
    {
        return 'المدارس';
    }

    public static function singularLabel(): string
    {
        return 'مدرسة';
    }

    /**
     * Build an "index" query for the given resource.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @param  \Illuminate\Database\Eloquent\Builder  $query
     * @return \Illuminate\Database\Eloquent\Builder
     */
    public static function indexQuery(NovaRequest $request, $query): Builder
    {
        // Eager load the count of assets for performance
        return $query->withCount('assets');
    }

    /**
     * Get the fields displayed by the resource.
     *
     * @return array<int, \Laravel\Nova\Fields\Field>
     */
    public function fields(NovaRequest $request): array
    {
        return [
            ID::make()->sortable(),

            Text::make('اسم المدرسة', 'name')
                ->sortable()
                ->rules('required', 'string', 'max:255'),
                
            Text::make('موقع المدرسة', 'location')
                ->sortable()
                ->rules('required', 'string', 'max:255'),
                
            Text::make('الرقم الوزاري', 'ministerial_number')
                ->sortable()
                ->rules('required', 'string', 'max:255')
                ->creationRules('unique:schools,ministerial_number')
                ->updateRules('unique:schools,ministerial_number,{{resourceId}}'),

            Text::make('إجمالي الأصول', 'assets_count')
                ->displayUsing(function ($count) {
                    if ($count === null) {
                        return '-';
                    }
                    // Style for the colored box
                    $bgColor = '#4299e1'; // Tailwind blue-500
                    $textColor = 'white';
                    $style = sprintf(
                        'display: inline-block; background-color: %s; color: %s; padding: 0.25rem 0.6rem; border-radius: 0.25rem; font-weight: bold; font-size: 0.875rem; min-width: 25px; text-align: center;',
                        $bgColor,
                        $textColor
                    );
                    return "<span style='{$style}'>{$count}</span>";
                })
                ->asHtml() // Render the output as HTML
                ->sortable() 
                ->exceptOnForms(), // Show on index/detail, hide on forms

            BelongsTo::make('أنشئ بواسطة', 'creator', User::class)
                ->exceptOnForms()
                ->searchable(),

            HasMany::make('الأصول', 'assets', Asset::class),
        ];
    }

    /**
     * Get the URI key for the resource.
     *
     * @return string
     */
    public static function uriKey(): string
    {
        return 'schools';
    }

    /**
     * Get the redirection destination after resource creation.
     *
     * @param  \Laravel\Nova\Http\Requests\NovaRequest  $request
     * @param  \Laravel\Nova\Resource  $resource
     * @return string
     */
    public static function redirectAfterCreate(NovaRequest $request, $resource): string
    {
        // Redirect to the detail page of the newly created school
        return '/resources/' . static::uriKey() . '/' . $resource->getKey();
    }

    /**
     * Get the cards available for the resource.
     *
     * @return array<int, \Laravel\Nova\Card>
     */
    public function cards(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @return array<int, \Laravel\Nova\Filters\Filter>
     */
    public function filters(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @return array<int, \Laravel\Nova\Lenses\Lens>
     */
    public function lenses(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the actions available for the resource.
     *
     * @return array<int, \Laravel\Nova\Actions\Action>
     */
    public function actions(NovaRequest $request): array
    {
        return [];
    }
}
