<?php

namespace App\Nova;

use App\Nova\Actions\ImportUsersAction;
use App\Nova\Filters\UserRoleFilter;
use App\Nova\Lenses\RecentlyCreatedUsers;
use Illuminate\Http\Request;
use Laravel\Nova\Actions\ExportAsCsv;
use Laravel\Nova\Auth\PasswordValidationRules;
use Laravel\Nova\Fields\BelongsToMany;
use Laravel\Nova\Fields\Avatar;
use Laravel\Nova\Fields\Image;
use Laravel\Nova\Fields\ID;
use Laravel\Nova\Fields\Password;
use Laravel\Nova\Fields\Text;
use Laravel\Nova\Fields\Boolean;
use Laravel\Nova\Http\Requests\NovaRequest;

class User extends Resource
{
    use PasswordValidationRules;

    /**
     * The model the resource corresponds to.
     *
     * @var class-string<\App\Models\User>
     */
    public static $model = \App\Models\User::class;

    /**
     * The single value that should be used to represent the resource when being displayed.
     *
     * @var string
     */
    public static $title = 'name';

    /**
     * ترتيب الموارد في القائمة الجانبية.
     *
     * @var int
     */
    public static $priority = 1;

    /**
     * تسمية المورد في القائمة الجانبية.
     *
     * @return string
     */
    public static function label(): string
    {
        return 'المستخدمين';
    }

    /**
     * تسمية المورد في القائمة الجانبية (المفرد).
     *
     * @return string
     */
    public static function singularLabel(): string
    {
        return 'مستخدم';
    }
    
    /**
     * التحقق مما إذا كان المستخدم مصرح له برؤية مورد المستخدمين.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public static function authorizedToViewAny(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('users.view');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له برؤية المستخدم.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToView(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('users.view');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بإنشاء مستخدم.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public static function authorizedToCreate(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('users.create');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بتحديث المستخدم.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToUpdate(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('users.edit');
    }

    /**
     * التحقق مما إذا كان المستخدم مصرح له بحذف المستخدم.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return bool
     */
    public function authorizedToDelete(Request $request): bool
    {
        return $request->user() && $request->user()->hasPermission('users.delete');
    }

    /**
     * The columns that should be searched.
     *
     * @var array
     */
    public static $search = [
        'id', 'name', 'email',
    ];

    /**
     * Indicates whether the resource should automatically poll for new resources.
     *
     * @var bool
     */
    public static $polling = true;

    /**
     * The interval at which Nova should poll for new resources.
     *
     * @var int
     */
    public static $pollingInterval = 5; // Poll every 5 seconds

    /**
     * Indicates whether to show the polling toggle button inside Nova.
     *
     * @var bool
     */
    public static $showPollingToggle = true; // Show the toggle button

    /**
     * The visual style used for the table. Available options are 'tight' and 'default'.
     *
     * @var string
     */
    public static $tableStyle = 'tight'; // Apply the tight table style

    /**
     * Get the fields displayed by the resource.
     *
     * @return array<int, \Laravel\Nova\Fields\Field|\Laravel\Nova\Panel|\Laravel\Nova\ResourceTool|\Illuminate\Http\Resources\MergeValue>
     */
    public function fields(NovaRequest $request): array
    {
        return [
            ID::make()->sortable(),

            Image::make('الصورة الرمزية', 'avatar_url')
                ->disk('public')
                ->maxWidth(50)
                ->rounded()
                ->exceptOnForms(),

            Avatar::make('الصورة الرمزية', 'avatar')
                ->disk('public')
                ->path('user-avatars')
                ->nullable()
                ->prunable()
                ->onlyOnForms(),

            Text::make('الاسم', 'name')
                ->sortable()
                ->rules('required', 'max:255'),

            Text::make('البريد الإلكتروني', 'email')
                ->sortable()
                ->rules('required', 'email', 'max:254')
                ->creationRules('unique:users,email')
                ->updateRules('unique:users,email,{{resourceId}}'),

            Boolean::make('نشط', 'is_active')
                ->sortable()
                ->rules('required', 'boolean'),

            Boolean::make('التحقق الثنائي', fn () => ! is_null($this->two_factor_confirmed_at))
                ->onlyOnIndex()
                ->sortable(),

            Password::make('كلمة المرور', 'password')
                ->onlyOnForms()
                ->creationRules($this->passwordRules())
                ->updateRules($this->optionalPasswordRules()),

            BelongsToMany::make('الإدارات', 'departments', Department::class)
                ->searchable(),

            BelongsToMany::make('الأدوار', 'roles', Role::class),
        ];
    }

    /**
     * Get the cards available for the request.
     *
     * @return array<int, \Laravel\Nova\Card>
     */
    public function cards(NovaRequest $request): array
    {
        return [];
    }

    /**
     * Get the filters available for the resource.
     *
     * @return array<int, \Laravel\Nova\Filters\Filter>
     */
    public function filters(NovaRequest $request): array
    {
        return [
            new UserRoleFilter(),
        ];
    }

    /**
     * Get the lenses available for the resource.
     *
     * @return array<int, \Laravel\Nova\Lenses\Lens>
     */
    public function lenses(NovaRequest $request): array
    {
        return [
            new RecentlyCreatedUsers(),
        ];
    }

    /**
     * Get the actions available for the resource.
     *
     * @return array<int, \Laravel\Nova\Actions\Action>
     */
    public function actions(NovaRequest $request): array
    {
        return [
            (new ImportUsersAction)->standalone(),

            new \App\Nova\Actions\DisableTwoFactorAuthentication,

            ExportAsCsv::make()->nameable(),
        ];
    }
}
