<?php

namespace App\Providers;

use App\Models\User;
use App\Nova\Asset;
use App\Nova\AssetBrand;
use App\Nova\AssetCategory;
use App\Nova\Dashboards\Main;
use App\Nova\Department;
use App\Nova\Permission;
use App\Nova\Role;
use App\Nova\School;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Gate;
use Laravel\Fortify\Features;
use Laravel\Nova\Menu\Menu;
use Laravel\Nova\Menu\MenuItem;
use Laravel\Nova\Menu\MenuSection;
use Laravel\Nova\Nova;
use Laravel\Nova\NovaApplicationServiceProvider;

class NovaServiceProvider extends NovaApplicationServiceProvider
{
    /**
     * Bootstrap any application services.
     */
    public function boot(): void
    {
        parent::boot();

        //
    /**تحويل المشروع الى RTL الخاص بمشروع شركة تطوير لتقنيات التعليم */
        Nova::enableRTL();

        // Define the custom main menu
        Nova::mainMenu(function (Request $request) {
            return [
                // Main Dashboard Link
                MenuSection::dashboard(Main::class)->icon('chart-bar'),

                // Settings/User Management Group
                MenuSection::make('إعدادات النظام', [
                    MenuItem::resource(\App\Nova\User::class),
                    MenuItem::resource(Role::class),
                    MenuItem::resource(Permission::class),
                    MenuItem::resource(Department::class),
                ])->icon('cog')->collapsable(),

                MenuSection::make('إدارة المدارس', [
                    MenuItem::resource(School::class),
                    // Add other school-related resources here
                ])->icon('building-office')->collapsable(), // New section for Schools
                
                MenuSection::make('إدارة الأصول', [
                    MenuItem::resource(Asset::class), // Added Asset resource here
                    MenuItem::resource(AssetBrand::class),
                    MenuItem::resource(AssetCategory::class),
                ])->icon('archive-box')->collapsable(),
            ];
        });
    }

    /**
     * Register the configurations for Laravel Fortify.
     */
    protected function fortify(): void
    {
        Nova::fortify()
            ->features([
                Features::updatePasswords(),
                // Features::emailVerification(),
                Features::twoFactorAuthentication(['confirm' => true, 'confirmPassword' => true]),
            ])
            ->register();
    }

    /**
     * Register the Nova routes.
     */
    protected function routes(): void
    {
        Nova::routes()
            ->withAuthenticationRoutes(default: true)
            ->withPasswordResetRoutes()
            ->withoutEmailVerificationRoutes()
            ->register();
    }

    /**
     * Register the Nova gate.
     *
     * This gate determines who can access Nova in non-local environments.
     */
    protected function gate(): void
    {
        Gate::define('viewNova', function (User $user = null) {
            // تحقق من وجود المستخدم
            if (!$user) {
                return false;
            }
            
            // فقط المستخدمين الذين لديهم دور 'admin' أو 'editor' يمكنهم الوصول إلى النوفا
            // أو المستخدمين الذين لديهم أي صلاحية من صلاحيات المستخدمين أو الأدوار أو الصلاحيات
            return $user->hasRole(['admin', 'editor']) || 
                   $user->hasPermission('users.view') || 
                   $user->hasPermission('roles.view') || 
                   $user->hasPermission('permissions.view');
        });
    }

    /**
     * Get the dashboards that should be listed in the Nova sidebar.
     *
     * @return array<int, \Laravel\Nova\Dashboard>
     */
    protected function dashboards(): array
    {
        return [
            new \App\Nova\Dashboards\Main,
        ];
    }

    /**
     * Get the tools that should be listed in the Nova sidebar.
     *
     * @return array<int, \Laravel\Nova\Tool>
     */
    public function tools(): array
    {
        return [];
    }

    /**
     * Register the Nova resources.
     *
     * @return void
     */
    protected function resources(): void
    {
        Nova::resourcesIn(app_path('Nova'));

        Nova::resources([
            \App\Nova\User::class,
            Role::class,
            Permission::class,
            Department::class,
            School::class, // Register School
            Asset::class, // Register Asset
            AssetBrand::class,
            AssetCategory::class,
        ]);
    }

    /**
     * Register any application services.
     */
    public function register(): void
    {
        parent::register();

        //
    }
}
